from logging import Logger
from typing import Any

from botocore.retries import bucket as bucket
from botocore.retries import standard as standard
from botocore.retries import throttling as throttling

logger: Logger = ...

def register_retry_handler(client: Any) -> Any: ...

class ClientRateLimiter:
    def __init__(
        self,
        rate_adjustor: Any,
        rate_clocker: Any,
        token_bucket: Any,
        throttling_detector: Any,
        clock: Any,
    ) -> None: ...
    def on_sending_request(self, request: Any, **kwargs: Any) -> None: ...
    def on_receiving_response(self, **kwargs: Any) -> None: ...

class RateClocker:
    def __init__(self, clock: Any, smoothing: Any = ..., time_bucket_range: Any = ...) -> None: ...
    def record(self, amount: int = ...) -> Any: ...
    @property
    def measured_rate(self) -> Any: ...



from typing import Any, Mapping, Optional

from botocore.awsrequest import create_request_object as create_request_object
from botocore.awsrequest import prepare_request_dict as prepare_request_dict
from botocore.compat import OrderedDict as OrderedDict
from botocore.exceptions import UnknownClientMethodError as UnknownClientMethodError
from botocore.exceptions import UnknownSignatureVersionError as UnknownSignatureVersionError
from botocore.exceptions import UnsupportedSignatureVersionError as UnsupportedSignatureVersionError
from botocore.hooks import BaseEventHooks
from botocore.model import ServiceId
from botocore.utils import datetime2timestamp as datetime2timestamp

class RequestSigner:
    def __init__(
        self,
        service_id: ServiceId,
        region_name: str,
        signing_name: str,
        signature_version: str,
        credentials: Any,
        event_emitter: BaseEventHooks,
        auth_token: Optional[str] = ...,
    ) -> None: ...
    @property
    def region_name(self) -> str: ...
    @property
    def signature_version(self) -> str: ...
    @property
    def signing_name(self) -> str: ...
    def handler(
        self, operation_name: Optional[str] = ..., request: Optional[Any] = ..., **kwargs: Any
    ) -> Any: ...
    def sign(
        self,
        operation_name: str,
        request: Any,
        region_name: Optional[str] = ...,
        signing_type: str = ...,
        expires_in: Optional[Any] = ...,
        signing_name: Optional[str] = ...,
    ) -> None: ...
    def get_auth_instance(
        self,
        signing_name: str,
        region_name: str,
        signature_version: Optional[str] = ...,
        **kwargs: Any,
    ) -> Any: ...
    get_auth: Any = ...
    def generate_presigned_url(
        self,
        request_dict: Mapping[str, Any],
        operation_name: str,
        expires_in: int = ...,
        region_name: Optional[str] = ...,
        signing_name: Optional[str] = ...,
    ) -> Any: ...

class CloudFrontSigner:
    key_id: Any = ...
    rsa_signer: Any = ...
    def __init__(self, key_id: str, rsa_signer: Any) -> None: ...
    def generate_presigned_url(
        self, url: str, date_less_than: Optional[Any] = ..., policy: Optional[Any] = ...
    ) -> Any: ...
    def build_policy(
        self,
        resource: Any,
        date_less_than: Any,
        date_greater_than: Optional[Any] = ...,
        ip_address: Optional[Any] = ...,
    ) -> Any: ...

def add_generate_db_auth_token(class_attributes: Any, **kwargs: Any) -> None: ...
def generate_db_auth_token(
    self: Any, DBHostname: Any, Port: Any, DBUsername: Any, Region: Optional[Any] = ...
) -> Any: ...

class S3PostPresigner:
    def __init__(self, request_signer: Any) -> None: ...
    def generate_presigned_post(
        self,
        request_dict: Mapping[str, Any],
        fields: Optional[Any] = ...,
        conditions: Optional[Any] = ...,
        expires_in: int = ...,
        region_name: Optional[str] = ...,
    ) -> Any: ...

def add_generate_presigned_url(class_attributes: Any, **kwargs: Any) -> None: ...
def generate_presigned_url(
    self: Any,
    ClientMethod: Any,
    Params: Optional[Any] = ...,
    ExpiresIn: int = ...,
    HttpMethod: Optional[Any] = ...,
) -> Any: ...
def add_generate_presigned_post(class_attributes: Any, **kwargs: Any) -> None: ...
def generate_presigned_post(
    self: Any,
    Bucket: Any,
    Key: Any,
    Fields: Optional[Any] = ...,
    Conditions: Optional[Any] = ...,
    ExpiresIn: int = ...,
) -> Any: ...



import logging
from typing import IO, Any, Dict, Iterator, Mapping, Optional, Sequence, Type, Union

from awscrt import checksums as crt_checksums
from botocore.awsrequest import AWSHTTPResponse
from botocore.compat import HAS_CRT as HAS_CRT
from botocore.exceptions import AwsChunkedWrapperError as AwsChunkedWrapperError
from botocore.exceptions import FlexibleChecksumError as FlexibleChecksumError
from botocore.model import OperationModel
from botocore.response import StreamingBody as StreamingBody
from botocore.utils import conditionally_calculate_md5 as conditionally_calculate_md5
from botocore.utils import determine_content_length as determine_content_length

logger: logging.Logger

class BaseChecksum:
    def update(self, chunk: Union[bytes, bytearray]) -> None: ...
    def digest(self) -> bytes: ...
    def b64digest(self) -> str: ...
    def handle(self, body: Union[bytes, bytearray, IO[Any]]) -> str: ...

class Crc32Checksum(BaseChecksum):
    def __init__(self) -> None: ...

class CrtCrc32Checksum(BaseChecksum):
    def __init__(self) -> None: ...

class CrtCrc32cChecksum(BaseChecksum):
    def __init__(self) -> None: ...

class Sha1Checksum(BaseChecksum):
    def __init__(self) -> None: ...

class Sha256Checksum(BaseChecksum):
    def __init__(self) -> None: ...

class AwsChunkedWrapper:
    def __init__(
        self,
        raw: IO[Any],
        checksum_cls: Optional[Type[BaseChecksum]] = ...,
        checksum_name: str = ...,
        chunk_size: Optional[int] = ...,
    ) -> None: ...
    def seek(self, offset: int, whence: int = ...) -> None: ...
    def read(self, size: Optional[int] = ...) -> bytes: ...
    def __iter__(self) -> Iterator[bytes]: ...

class StreamingChecksumBody(StreamingBody):
    def __init__(
        self, raw_stream: IO[Any], content_length: int, checksum: BaseChecksum, expected: str
    ) -> None: ...
    def read(self, amt: Optional[int] = ...) -> bytes: ...

def resolve_checksum_context(
    request: Dict[str, Any], operation_model: OperationModel, params: Mapping[str, Any]
) -> None: ...
def resolve_request_checksum_algorithm(
    request: Dict[str, Any],
    operation_model: OperationModel,
    params: Mapping[str, Any],
    supported_algorithms: Optional[Sequence[str]] = ...,
) -> None: ...
def apply_request_checksum(request: Dict[str, Any]) -> None: ...
def resolve_response_checksum_algorithms(
    request: Dict[str, Any],
    operation_model: OperationModel,
    params: Mapping[str, Any],
    supported_algorithms: Optional[Sequence[str]] = ...,
) -> None: ...
def handle_checksum_body(
    http_response: AWSHTTPResponse,
    response: Dict[str, Any],
    context: Mapping[str, Any],
    operation_model: OperationModel,
) -> None: ...
